<?php

namespace Drupal\eck_site_settings;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Drupal\eck\EckEntityBundleInterface;
use Drupal\eck\EckEntityInterface;
use Drupal\eck\EckEntityTypeInterface;

/**
 * A service for loading settings related entity types, bundles and entities.
 */
class SettingsRepository implements SettingsRepositoryInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type repository.
   *
   * @var \Drupal\Core\Entity\EntityTypeRepositoryInterface
   */
  protected $entityTypeRepository;

  /**
   * SettingsRepository constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeRepositoryInterface $entityTypeRepository
   *   The entity type repository.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    EntityTypeRepositoryInterface $entityTypeRepository
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityTypeRepository = $entityTypeRepository;
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityTypes(): array {
    $storage = $this->entityTypeManager
      ->getStorage('eck_entity_type');
    $ids = $storage->getQuery()
      ->condition('third_party_settings.eck_site_settings.enabled', TRUE)
      ->sort('label')
      ->accessCheck(FALSE)
      ->execute();

    if ($ids === []) {
      return [];
    }

    return $storage->loadMultiple($ids);
  }

  /**
   * {@inheritdoc}
   */
  public function getBundles(EckEntityTypeInterface $entityType): array {
    if (!$this->entityTypeManager->hasDefinition($entityType->id())) {
      return [];
    }

    $definition = $this->entityTypeManager
      ->getDefinition($entityType->id());
    $storage = $this->entityTypeManager
      ->getStorage($definition->getBundleEntityType());

    return $storage->loadMultiple();
  }

  /**
   * {@inheritdoc}
   */
  public function isSetting(string $entityTypeId): bool {
    $storage = $this->entityTypeManager
      ->getStorage('eck_entity_type');
    $query = $storage->getQuery()
      ->condition('id', $entityTypeId)
      ->condition('third_party_settings.eck_site_settings.enabled', TRUE)
      ->accessCheck(FALSE);

    return (bool) $query->count()
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function getSetting(string $bundle, string $entityTypeId = 'settings'): EckEntityInterface {
    $definition = $this->entityTypeManager
      ->getDefinition($entityTypeId);
    $storage = $this->entityTypeManager
      ->getStorage($entityTypeId);
    $ids = $storage->getQuery()
      ->condition($definition->getKey('bundle'), $bundle)
      ->range(0, 1)
      ->accessCheck(FALSE)
      ->execute();

    if ($ids === []) {
      return $this->createEntity($bundle, $entityTypeId);
    }

    return $storage->load(reset($ids));
  }

  /**
   * {@inheritdoc}
   */
  public function getSettingByClass(string $className): ?EckEntityInterface {
    $entityTypeId = $this->entityTypeRepository
      ->getEntityTypeFromClass($className);
    $storage = $this->entityTypeManager
      ->getStorage($entityTypeId);

    if (!method_exists($storage, 'getBundleFromClass')) {
      throw new \LogicException('Entity bundle classes were introduced in Drupal 9.3.0-alpha1. Please update to be able to use this method.');
    }

    $bundle = $storage->getBundleFromClass($className);
    if ($bundle === NULL) {
      return NULL;
    }

    return $this->getSetting($bundle);
  }

  /**
   * Creates a new settings entity.
   *
   * @param string $bundle
   *   The bundle.
   * @param string $entityTypeId
   *   The entity type ID.
   *
   * @return \Drupal\eck\EckEntityInterface
   *   The settings entity.
   *
   * @throws \InvalidArgumentException
   *   Thrown when the bundle does not exist.
   */
  protected function createEntity(string $bundle, string $entityTypeId): EckEntityInterface {
    $definition = $this->entityTypeManager
      ->getDefinition($entityTypeId);
    $storage = $this->entityTypeManager
      ->getStorage($entityTypeId);
    $bundleEntity = $this->entityTypeManager
      ->getStorage($definition->getBundleEntityType())
      ->load($bundle);

    if (!$bundleEntity instanceof EckEntityBundleInterface) {
      throw new \InvalidArgumentException(sprintf('The bundle "%s" does not exist.', $bundle));
    }

    $entity = $storage->create([
      $definition->getKey('bundle') => $bundle,
      'title' => $bundleEntity->label(),
    ]);
    $entity->save();

    return $entity;
  }

}
